/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __RedirectStreams_hh__
#define __RedirectStreams_hh__
#include "bias_config.h" 

#include <iostream>
#include <fstream>
#include <sstream>

namespace BIAS{
    /** @class RedirectStreams
    @brief used to redirect stdout and stderr into files or to silence them
    */
    class BIASDebug_EXPORT RedirectStreams {

    public:
        virtual ~RedirectStreams(){};

        /** @brief empty useless constructor, does nothing */
        inline RedirectStreams(){};

        // required due to members JW
        RedirectStreams(const RedirectStreams &src){
            (*this) = src;
        };


        #include <Common/BIASpragmaStart.hh>
        // required due to members JW
        RedirectStreams & operator=(const RedirectStreams &src){
            BIASERR("not implemented due to streams. (jw)");
            return *this;
        };
        #include <Common/BIASpragmaEnd.hh>


        /** @brief direct all stdout output into a file */
        inline void RedirectCout(char* file);

        /** @brief direct all stderr output into a file */
        inline void RedirectCerr(char* file);

        inline void RedirectCout(const std::string& file) 
        { RedirectCout((char *)file.c_str()); }

        inline void RedirectCerr(const std::string& file) 
        { RedirectCerr((char *)file.c_str()); } 

        // no /dev/null on windows
#ifndef WIN32
        /** @brief call this to silence cout */
        inline void Cout2Null() {
            RedirectCout("/dev/null");
        }

        /** @brief call this to silence cerr */
        inline void Cerr2Null() {
            RedirectCerr("/dev/null");
        }
#endif

        /** @brief re-enables backupped standard output for cout */
        inline void ResetCout();
        /** @brief re-enables backupped standard output for cerr */
        inline void ResetCerr();

    protected:

        std::streambuf *OutBuf_, *ErrBuf_, *RedirOut_, *RedirErr_;

        // redirecting cout, cerr to avoid output
        std::ofstream OutStr_, ErrStr_;

    }; // class RedirectStreams

    inline void RedirectStreams::RedirectCout(char* file)
    {
        OutStr_.open (file);
        RedirOut_ = OutStr_.rdbuf();
        OutBuf_   = std::cout.rdbuf(); 
        std::cout.rdbuf(RedirOut_); 
    }

    inline void RedirectStreams::RedirectCerr(char* file)
    {
        ErrStr_.open (file);
        RedirErr_ = ErrStr_.rdbuf();
        ErrBuf_   = std::cerr.rdbuf();
        std::cerr.rdbuf(RedirErr_);
    }

    inline void RedirectStreams::ResetCout()
    {
        std::cout.rdbuf(OutBuf_);
        OutStr_.close();
    }

    inline void RedirectStreams::ResetCerr()
    {
        std::cerr.rdbuf(ErrBuf_);
        ErrStr_.close();
    }


} // namespace


#endif //__BIASRedirectStreams_hh__

