/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _Normalization_hh_
#define _Normalization_hh_

#include "bias_config.h" 
#include <Common/BIASpragmaStart.hh>
#include <vector>


namespace BIAS {
  // forward declarations
  class KMatrix;
  class HomgPoint2D;
  template <class T> class BIASMathBase_EXPORT Matrix3x3;

  /** @class Normalization
      @ingroup g_geometry
      @brief class Normalization Functions, e.g. Hartley
      @author koeser 07/2004 **/
  class BIASGeometryBase_EXPORT Normalization 
  {
  public:
    
    /**	default constructor, does nothing **/
    inline Normalization() {};
   
    /** @brief linearly transform Points with 3x3 matrix, such that they
        have mean zero and stddev sqrt(2)
        @param Points original (homogenized) points
        @param NormPoints normalized (homogenized) points correponding to Points
        @param Normalization matrix used for normalization
        @author koeser */
    int Hartley(const std::vector<HomgPoint2D>& Points,
                std::vector<HomgPoint2D>& NormPoints,
                KMatrix& Normalization) const;
  
  };	
    

} // namespace BIAS

#include <Common/BIASpragmaEnd.hh>

#endif

