/** This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Geometry/SimilarityTransform.hh>
#include <Math/SVD.hh>
#include <Geometry/RMatrixBase.hh>
#include <fstream>
#include <iostream>
#include <Math/Matrix3x4.hh>

using namespace BIAS;
using namespace std;


bool SimilarityTransform::
Load(const std::string& inputFile)
{
  ifstream ifs(inputFile.c_str());
  if(!ifs)
    return false;
  ifs>>*this;
  if(!ifs) {
    ifs.close();
    return false;
  }
  ifs.close();
  return true;
}

bool SimilarityTransform::
Save(const std::string& outputFile)
{
  ofstream ofs(outputFile.c_str());
  if(!ofs)
    return false;
  ofs<<*this;
  if(!ofs) {
    ofs.close();
    return false;
  }
  ofs.close();
  return true;
}


void SimilarityTransform::Dump()
{
  cout<<"Translation:"<<C_<<endl;

  RMatrixBase R;
  R.SetFromQuaternion(Q_);
  cout<<"Rotation Matrix: "<<R<<endl;
  
  Vector3<double> axis;
  double angle;
  
  R.GetRotationAxisAngle(axis, angle);
  cout<<" axis = "<<axis<<endl;
  cout<<"angle = "<<angle<<endl<<endl;
  cout<<"Matrix representation:"<<GetMatrix()<<endl;
}

Matrix4x4<double> SimilarityTransform::GetMatrix() const
{
  Matrix4x4<double> result;
  RMatrixBase R; 
  for (unsigned int r=0; r< 3; r++)
    result[r][r]= scale_;
  result[3][3] = 1.0;
  R.SetFromQuaternion(Q_);
  for (unsigned int r=0; r< 3; r++)
    for (unsigned int c=0; c< 3; c++)
      result[r][c]= R[r][c];
  result[0][2] = C_[0];
  result[1][2] = C_[1];
  result[2][2] = C_[2];
  return result;
}

const Matrix4x4<double> SimilarityTransform::GetTransformMatrix() const 
{
  BIAS::RMatrixBase R; R.SetFromQuaternion(GetQ());
  Matrix4x4<double> T; 
  T.SetIdentity();
  T*=scale_;
  T[3][3]=1.0;
  for (int i=0;i<3;i++) 
    for (int j=0;j<3;j++)
      T[i][j]=R[i][j];
  for (int j=0;j<3;j++)
    T[j][3]=C_[j];
  return T;
}
