/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#ifndef __PyramidImageInterface_hh__
#define __PyramidImageInterface_hh__

#include <Common/BIASpragmaStart.hh>

#include <string>
#include <iostream>

namespace BIAS 
{
  // forward declarations
  class ImageBase;
  template <class T> class Image;
  template <class T> class SharedPtr;
  class UUID;

  /** @class PyramidImageInterface
      @ingroup g_image
      @brief interface definition of all pyramid images

      @author woelk 02/2008 (c) www.vision-n.de */
  template <class StorageType>
  class BIASImage_EXPORT PyramidImageInterface
  {
  public:
    PyramidImageInterface() {};

    virtual ~PyramidImageInterface() {};

    virtual void Init(const Image<StorageType>& image, 
                      const unsigned py_size) = 0;

    virtual void InitFromImageBase(const ImageBase& image, 
                                   const unsigned py_size) = 0;

    virtual void Init(const unsigned int width, const unsigned int height, 
                      const unsigned int channelcount, 
                      const unsigned py_size) = 0;

    virtual void Init(const unsigned pyramid_size) = 0;

    virtual void Clear() = 0;

    virtual SharedPtr<Image<StorageType> >& 
      operator[](const unsigned index) = 0;

    virtual SharedPtr<const Image<StorageType> > 
      operator[](const unsigned index) const = 0;

    virtual StorageType 
      GetTrilinearImageValue(const double& x, const double& y,
                             const double& scale, int channel=0) const = 0;
    
    virtual StorageType 
      GetImageValue(const double& x, const double& y,
                    unsigned int scale, int channel=0) const = 0;

    virtual void GetSingleImage(Image<StorageType>& im) const = 0;

    virtual int SetROI(unsigned minx, unsigned miny, 
                       unsigned maxx, unsigned maxy) = 0;

    virtual bool IsEmpty() const = 0;
    
    /// deprecated interface
    inline bool empty() const { return IsEmpty(); }

    virtual unsigned Size() const = 0;

    /// deprecated interface
    inline unsigned size() const { return Size(); }

    virtual void SetZero() = 0;

    virtual int WriteImages(const std::string& prefix) = 0;
    
    
    /// deprecated interface
    inline unsigned GetPyramidSize() const { return Size(); };

    virtual void resize(const unsigned size) = 0;

    virtual double GetRescaleFactor() const = 0;

    virtual double GetPositionOffset() const = 0;

    virtual int Downsample() = 0;

    virtual void Dump(std::ostream& os = std::cout) const = 0;
  protected:
    

  };

} // namespace BIAS


#include <Common/BIASpragmaEnd.hh>

#endif
